using System;
using System.Collections;

using System.Linq;
using DevExpress.Data.Filtering;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Actions;
using DevExpress.ExpressApp.Editors;
using DevExpress.ExpressApp.SystemModule;
using Xafari;
using Xafari.Model;

namespace Solution2.Module.Controllers
{
    using System.Data.Objects;

    using Solution2.Module.BusinessObjects;

    public partial class UndoWizardForDeletedEntitiesViewController : ViewController
    {
        private DetailView detailView;
        public UndoWizardForDeletedEntitiesViewController()
        {
            InitializeComponent();
            TargetViewType = ViewType.DetailView;
            TargetObjectType = typeof(IDcSoftDelete);

            saDeletedUndo.TargetObjectType = typeof(IDcSoftDelete);
            saDeletedCancel.TargetObjectType = typeof(IDcSoftDelete);

            saDeletedUndo.Active.SetItemValue("NotDeleted", false);
            saDeletedCancel.Active.SetItemValue("NotDeleted", false);

            saDeletedUndo.Model.Index = 0;
            saDeletedCancel.Model.Index = 1;
        }
        protected override void OnActivated()
        {
            base.OnActivated();
            
            detailView = View as DetailView;
            if (detailView != null)
            {
                if ((detailView.CurrentObject as IDcSoftDelete) == null
                    || !((IDcSoftDelete)detailView.CurrentObject).IsDeleted) return;
               
                detailView.ViewEditMode = ViewEditMode.View;
                detailView.AllowEdit.SetItemValue("IsDeleted", false);
                detailView.AllowDelete.SetItemValue("IsDeleted", false);
                detailView.AllowNew.SetItemValue("IsDeleted", false);
                detailView.Tag = "NoJS";
            }

            Frame.GetController<RefreshController>().RefreshAction.Active.SetItemValue("IsDeleted", false);

            saDeletedUndo.Active.RemoveItem("NotDeleted");
            saDeletedCancel.Active.RemoveItem("NotDeleted");

            saDeletedUndo.Execute += SaDeletedUndoExecute;
            saDeletedCancel.Execute += SaDeletedCancelExecute;
        }
        
        protected override void OnDeactivated()
        {
            base.OnDeactivated();
            if (saDeletedUndo != null)
                saDeletedUndo.Execute -= SaDeletedUndoExecute;

            if (saDeletedCancel != null)
                saDeletedCancel.Execute -= SaDeletedCancelExecute;
        }

        private void SaDeletedUndoExecute(object sender, SimpleActionExecuteEventArgs e)
        {
            var currentObject = detailView.CurrentObject;
            var currentObjectType = ObjectContext.GetObjectType(currentObject.GetType());

            if (!UndoActionCheckCodeUnique(currentObjectType, currentObject))
            {
                SetupWizardShowViewParameters(e.ShowViewParameters, Application.Model.Xafari().Wizards.GetWizard("waUndoDeleted"));
                return;
            }

            UndoActionExecute(currentObjectType, currentObject);

            Frame.GetController<ModificationsController>().CancelAction.Active.RemoveItem("EditMode");
            Frame.GetController<ModificationsController>().CancelAction.DoExecute();
        }

        private void SaDeletedCancelExecute(object sender, SimpleActionExecuteEventArgs e)
        {
            Frame.GetController<ModificationsController>().CancelAction.Active.RemoveItem("EditMode");
            Frame.GetController<ModificationsController>().CancelAction.DoExecute();
        }

        private bool UndoActionCheckCodeUnique(Type objectType, object objectEntity)
        {
            using (var objectSpace = Application.CreateObjectSpace())
            {
                var localObject = objectEntity as IDcCode;
                if (localObject == null)
                {
                    return true;
                }

                var results = objectSpace.GetObjects(
                    objectType,
                    CriteriaOperator.Parse(@"StartsWith(Code, '{localObject.Code}')"));
                
                //in my situation "!(results.Count > 0)" returns true
                //so I left the (results.Count > 0) option for some time
                return (results.Count > 0);

            }
        }

        private void UndoActionExecute(Type objectType, object objectEntity)
        {
            var dcSoftDelete = objectEntity as IDcSoftDelete;
            if (dcSoftDelete == null || !dcSoftDelete.IsDeleted || objectType == null)
            {
                throw new ArgumentException("UndoActionExecute Argument Exception");
            }

            try
            {
                var typeInfo = ObjectSpace.TypesInfo.FindTypeInfo(objectType);
                foreach (var memberInfo in typeInfo.Members.Where(x => x.IsAssociation))
                {
                    if (memberInfo.AssociatedMemberInfo.IsList) continue;

                    var referenceMember = memberInfo.GetValue(objectEntity) as IDcSoftDelete;
                    if (!memberInfo.IsList && referenceMember != null && referenceMember.IsDeleted)
                    {
                        referenceMember.IsDeleted = false;
                    }

                    var collectionMember = memberInfo.GetValue(objectEntity) as ICollection;
                    if (!memberInfo.IsList || collectionMember == null) continue;

                    foreach (var childReferenceMember in collectionMember.Cast<object>().OfType<IDcSoftDelete>().Where(childReferenceMember => childReferenceMember.IsDeleted))
                    {
                        childReferenceMember.IsDeleted = false;
                    }
                }
            }
            catch (Exception exp)
            {
                throw new InvalidOperationException("UndoActionExecute Operation Exception", exp);
            }
            
            dcSoftDelete.IsDeleted = false;
            ObjectSpace.CommitChanges();
        }

        private void SetupWizardShowViewParameters(ShowViewParameters showViewParameters, IModelWizard wizard)
        {
            var objectSpace = Application.CreateObjectSpace();
            
            var dummyObject = objectSpace.CreateObject<TestClass1>();
            var detailView1 = View as DetailView;
            if (detailView1 != null)
            {
                var dcCode = detailView1.CurrentObject as IDcCode;
                if (dcCode != null)
                    dummyObject.Code = dcCode.Code;
            }

            var controller = XafariModule.CreateWizardDialogController(Application);
            controller.Model = wizard;

            controller.AcceptAction.Active.SetItemValue("test",true);

            controller.CurrentObject = (object)dummyObject;
            controller.Accepting += UndoDeletedWizardOnAccepting;
            controller.CustomNextStep += ControllerOnCustomNextStep;

            var createdView = Application.CreateDetailView(objectSpace, wizard[0].View, true, dummyObject);
            createdView.ViewEditMode = ViewEditMode.Edit;
            createdView.Tag = "NoJS";

            showViewParameters.Controllers.AddRange(new[] { controller });
            showViewParameters.Context = wizard.TemplateContext;
            showViewParameters.CreatedView = createdView;
            showViewParameters.TargetWindow = TargetWindow.NewModalWindow;
            showViewParameters.NewWindowTarget = NewWindowTarget.Default;
        }

        private void ControllerOnCustomNextStep(object sender, CustomNextStepEventArgs e)
        {
            if (e.CurrentStep.Index != 1) return;
            e.NextStep = UndoActionCheckCodeUnique(
                ObjectContext.GetObjectType(detailView.CurrentObject.GetType()),
                e.WizardObject<TestClass1>()) ? e.CurrentStep.Wizard[2] : e.CurrentStep.Wizard[0];
        }

        private void UndoDeletedWizardOnAccepting(object sender, DialogControllerAcceptingEventArgs e)
        {
            var wizardDialogController = sender as WizardDialogController;
            if (wizardDialogController != null)
            {
                wizardDialogController.Accepting -= UndoDeletedWizardOnAccepting;
            }

            var currentObject = detailView.CurrentObject;
            var currentObjectType = ObjectContext.GetObjectType(currentObject.GetType());

            var dummyObject = e.AcceptActionArgs.CurrentObject as TestClass1;
            if (dummyObject != null)
            {
                var dcCode = currentObject as IDcCode;
                if (dcCode != null)
                {
                    dcCode.Code = dummyObject.Code;
                }
            }

            UndoActionExecute(currentObjectType, currentObject);

            Frame.GetController<ModificationsController>().CancelAction.Active.RemoveItem("EditMode");
            Frame.GetController<ModificationsController>().CancelAction.DoExecute();
        }
        
    }
    
}
